-- cell voltage calculator /  JZ vers 004

local inputs     = { {"Bat.volt", SOURCE}, {"Play", VALUE, 0, 1, 0} }
local outputs    = { "Vcel" }
local wait_end   = 0
local cell_count = 1
local state
local filtered_voltage = 0   
local batt_on          = 0  

--state functions forward declaration
local wait, no_battery, wait_to_stabilize, calc, done

-------------------------------------------------------

function no_battery()
   -- wait for battery
   batt_on    = 0

   if filtered_voltage > 3 then
       batt_on    = 1
       state      = wait_to_stabilize
       wait_end   = getTime() + 200
   end  --  end if

end  -- no_battery()

-------------------------------------------------------

function wait_to_stabilize()

   -- wait some time for battery voltage to stabilize

   if getTime() >= wait_end then
       state = calc
   end  --  end if

end  -- end wait_to_stabilize()

-------------------------------------------------------

function calc(play)

   -- calculate cell count  --this works up to 12 cells
   cell_count = math.ceil(filtered_voltage / 4.25)  

   -- play detected cell count

   if play > 0 then

       playNumber(cell_count, 0, 0)
       
       for i= 1,cell_count  do 

            playTone(1300, 200, 600 , 0)  
            -- play one beep per cell , just like an ESC 

       end -- end for
       
       playFile("/SOUNDS/en/actv.wav") -- wav says: "active"  
  
   end  --  end if

   state = done
   
end   -- end calc(play)

-------------------------------------------------------

function done()

   if filtered_voltage < 1 then
       state      = no_battery
       batt_on    = 0
       
   end  -- end if

end  -- end done()

-------------------------------------------------------

local function run(voltage, play)

   -- the following table of percentages has 121 percentage values ,
   -- starting from 3.0 V to 4.2 V , in steps of 0.01 V 

   local Percent_Table = 
	{0  , 1  , 1  ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 ,  1 , 
	 2  , 2  , 2  ,  2 ,  2 ,  2 ,  2 ,  2 ,  2 ,  2 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 ,  3 , 
	 4  , 4  , 4  ,  4 ,  4 ,  4 ,  4 ,  4 ,  5 ,  5 ,  5 ,  5 ,  5 ,  5 ,  6 ,  6 ,  6 ,  6 ,  6 ,  6 , 
	 7  , 7  , 7  ,  7 ,  8 ,  8 ,  9 ,  9 , 10 , 12 , 13 , 14 , 17 , 19 , 20 , 22 , 23 , 26 , 28 , 30 , 
	 33 , 36 , 39 , 42 , 45 , 48 , 51 , 54 , 57 , 58 , 60 , 62 , 64 , 66 , 67 , 69 , 70 , 72 , 74 , 75 , 
	 77 , 78 , 80 , 81 , 82 , 84 , 85 , 86 , 86 , 87 , 88 , 89 , 91 , 92 , 94 , 95 , 96 , 97 , 97 , 99 , 100  }

   filtered_voltage = filtered_voltage * 0.9  +  voltage * 0.1
   
   if state == nil then state = no_battery   end    --state initialization

   state(play)     -- call state function
   

   if cell_count > 0 then 

	local V_Cell      = 3
        local VB          = 0
	local table_index = 1

       	V_Cell            = filtered_voltage / cell_count 
        table_index       = math.floor( 100 * V_Cell - 298 )
	batt_on           = 1     

	-- table_index       = 100   -- test case , index = 100 should report 75% charge

	if table_index    > 120 then  table_index = 120 end  --## check for index bounds
	if table_index    <   1 then  table_index =   1 end

    	VB            =   Percent_Table[table_index]

        return( VB * 10.24 * batt_on )

   end  -- end if 

   return 0  ---  no error

end -- end run

-------------------------------------------------------

---  OpenTX LUA interface parameters declared here

return { run=run, input=inputs, output=outputs }

-------------------------------------------------------